diff -urN asterisk-10.5.1.orig/apps/app_externalivr.c asterisk-10.5.1/apps/app_externalivr.c
--- asterisk-10.5.1.orig/apps/app_externalivr.c	2012-04-19 09:04:21.000000000 -0700
+++ asterisk-10.5.1/apps/app_externalivr.c	2012-06-20 16:21:55.797607721 -0700
@@ -205,6 +205,7 @@
 {
 	struct ivr_localuser *u = state->u;
 	char *file_to_stream;
+	struct playlist_entry *entry;
 
 	u->abort_current_sound = 0;
 	u->playing_silence = 0;
@@ -221,10 +222,11 @@
 
 		if (!(state->stream = ast_openstream_full(u->chan, file_to_stream, u->chan->language, 1))) {
 			ast_chan_log(LOG_WARNING, u->chan, "File '%s' could not be opened: %s\n", file_to_stream, strerror(errno));
-			AST_LIST_LOCK(&u->playlist);
-			AST_LIST_REMOVE_HEAD(&u->playlist, list);
-			AST_LIST_UNLOCK(&u->playlist);
 			if (!u->playing_silence) {
+				/* if we can't open this file and it's not silence, we need to remove and free it */
+				if((entry = AST_LIST_REMOVE_HEAD(&u->playlist, list))) {
+					ast_free(entry);
+				}
 				continue;
 			} else {
 				break;
@@ -239,29 +241,43 @@
 {
 	struct ast_frame *f = NULL;
 	struct ivr_localuser *u = state->u;
-
-	if (u->abort_current_sound ||
-		(u->playing_silence && AST_LIST_FIRST(&u->playlist))) {
-		gen_closestream(state);
+	struct playlist_entry *finished = NULL;
+	/* avoid locking in the critical path */
+	if(u->abort_current_sound) {
 		AST_LIST_LOCK(&u->playlist);
+		gen_closestream(state);
 		gen_nextfile(state);
 		AST_LIST_UNLOCK(&u->playlist);
+	} else if(u->playing_silence) {
+		/* if we're playing silence, check the playlist */
+		AST_LIST_LOCK(&u->playlist);
+		if(AST_LIST_FIRST(&u->playlist)) {
+			gen_closestream(state);
+			gen_nextfile(state);
+		}
+		AST_LIST_UNLOCK(&u->playlist);
 	}
 
 	if (!(state->stream && (f = ast_readframe(state->stream)))) {
 		if (state->current) {
 			/* remove finished file from playlist */
-                        AST_LIST_LOCK(&u->playlist);
-                        AST_LIST_REMOVE_HEAD(&u->playlist, list);
+			AST_LIST_LOCK(&u->playlist);
+			AST_LIST_REMOVE_HEAD(&u->playlist, list);
+			finished = state->current;
+			state->current = NULL;
+			gen_nextfile(state);
                         AST_LIST_UNLOCK(&u->playlist);
 			/* add finished file to finishlist */
 			AST_LIST_LOCK(&u->finishlist);
-			AST_LIST_INSERT_TAIL(&u->finishlist, state->current, list);
+			AST_LIST_INSERT_TAIL(&u->finishlist, finished, list);
 			AST_LIST_UNLOCK(&u->finishlist);
-			state->current = NULL;
+		} else {
+			AST_LIST_LOCK(&u->playlist);
+			gen_nextfile(state);
+			AST_LIST_UNLOCK(&u->playlist);
 		}
-		if (!gen_nextfile(state))
-			f = ast_readframe(state->stream);
+		if(state->stream)
+			f = ast_readframe(state->stream); 
 	}
 
 	return f;
@@ -591,9 +607,11 @@
 	if (ser) {
 		ao2_ref(ser, -1);
 	}
+	AST_LIST_LOCK(&u->playlist);
 	while ((entry = AST_LIST_REMOVE_HEAD(&u->playlist, list))) {
 		ast_free(entry);
 	}
+	AST_LIST_UNLOCK(&u->playlist);
 	return res;
 }
 
@@ -662,15 +680,15 @@
  		exception = 0;
  
  		rchan = ast_waitfor_nandfds(&chan, 1, waitfds, (eivr_errors_fd) ? 2 : 1, &exception, &ready_fd, &ms);
- 
+
+		AST_LIST_LOCK(&u->finishlist); 
  		if (chan->_state == AST_STATE_UP && !AST_LIST_EMPTY(&u->finishlist)) {
- 			AST_LIST_LOCK(&u->finishlist);
  			while ((entry = AST_LIST_REMOVE_HEAD(&u->finishlist, list))) {
  				send_eivr_event(eivr_events, 'F', entry->filename, chan);
  				ast_free(entry);
  			}
- 			AST_LIST_UNLOCK(&u->finishlist);
  		}
+		AST_LIST_UNLOCK(&u->finishlist);
  
  		if (chan->_state == AST_STATE_UP && !(ast_check_hangup(chan)) && rchan) {
  			/* the channel has something */
